//==============================================================================
//
//  リーフォニーのためのサンプルスケッチ
//
//     用途  : ビーコン・アドバタイザー（発信側）
//		 　　　　なるべく簡単に！
//     MCU   : ATmega328P (3.3V /8MHz)
//     Arduino IDEのバージョン  : 1.8.13
//
//     リーフの構成
//       (1) AI01 4-Sensors 加速度センサを使用
//       (2) AC02 BLE Sugar BLE ビーコンで発信
//       (3) AP01 AVR MCU   MCU
//       (4) AZ01 USB       デバッグ（プログラム書き込み）用
//       (5) AV01 CR2032    コイン電池
//
//    元のプログラム : 
//    https://github.com/Leafony/leafony-beacon/tree/master/simple-scanner/Arduino/leafony_beacon
//    Copyright (c) 2019  Trillion-Node Engine Project
//
//    Released under the MIT license（ライセンス：MITライセンス）
//    https://opensource.org/licenses/mit-license.php//
//
//    Rev.1 2021/10/01 	modified by Yamagishi
//
//==============================================================================


//==============================================================================
// 定義
//==============================================================================

#include <MsTimer2.h>         // タイマ割り込みライブラリ
#include <Wire.h>             // I2C通信ライブラリ（センサとの通信）
#include <Adafruit_LIS3DH.h>  // 加速度センサライブラリ

#include "TBGLib.h"           // BLEライブラリ
#include <SoftwareSerial.h>   // ソフトウエア・シリアル通信ライブラリ


//------------------------------------------------------------------------------
// ビーコン送出時に埋め込むデバイス名
// 長さは必ず6文字
//------------------------------------------------------------------------------
//                     |123456|
char strDeviceName[] = "DCUT4a";


//------------------------------------------------------------------------------
// シリアルコンソールへのデバック出力
//    #define DEBUG = 出力あり
//　　// #define DEBUG = 出力なし（コメントアウトする）
//------------------------------------------------------------------------------
#define DEBUG


//------------------------------------------------------------------------------
// IOピンの名前定義
// ＊：今回使用したピン
//------------------------------------------------------------------------------
// PD port
//   ＊digital 0: PD0 = PCRX    (HW UART) デバッグ用 USB
//   ＊digital 1: PD1 = PCTX    (HW UART) デバッグ用 USB
//     digital 2: PD2 = INT0#
//     digital 3: PD3 = INT1#
//     digital 4: PD4 = RSV
//     digital 5: PD5 = CN3_D5
//   ＊digital 6: PD6 = DISCN
//   ＊digital 7: PD7 = BLSLP#
// PB port
//     digital 8: PB0 = UART2_RX (software UART)
//     digital 9: PB1 = UART2_TX (software UART)
//     digital 10:PB2 = SS#
//     digital 11:PB3 = MOSI
//     digital 12:PB4 = MISO
//   ＊digital 13:PB5 = SCK (LED)
//                PB6 = XTAL1
//                PB7 = XTAL2
// PC port
//     digital 14/ Analog0: PC0 = PIN24_D14
//   ＊digital 15/ Analog1: PC1 = BLETX (software UART)
//   ＊digital 16/ Analog2: PC2 = BLERX (software UART)
//     digital 17/ Analog3: PC3 = PIN27_D17
//   ＊digital 18/ SDA    : PC4 = SDA   (I2C)
//   ＊digital 19/ SCL    : PC5 = SCL   (I2C)
//     RESET              : PC6 = RESET#
//------------------------------------------------------------------------------
#define BLE_RESET_PIN			6
#define BLE_WAKEUP_PIN		7
#define LED_PIN						13
#define BLETX							15
#define BLERX							16


//------------------------------------------------------------------------------
// プログラム内で使用する定数定義
//------------------------------------------------------------------------------
//-----------------------------------------------
// ３軸加速度センサ I2Cアドレス
//-----------------------------------------------
#define LIS2DH_ADDRESS 0x19  // SD0/SA0 pin = VCC

//-----------------------------------------------
// MsTimer2のタイマー割り込み発生間隔(ms)
// loop()内でタイマ割り込みと連携
//-----------------------------------------------
#define LOOP_INTERVAL 125 // 125ms interval



//==============================================================================
// オブジェクト
//==============================================================================
//-----------------------------------------------
// 加速度センサ
//-----------------------------------------------
Adafruit_LIS3DH accel = Adafruit_LIS3DH();

//-----------------------------------------------
// BLE
//-----------------------------------------------
SoftwareSerial Serialble(BLERX, BLETX);
BGLib ble112((HardwareSerial *)&Serialble, 0, 0);


//==============================================================================
// プログラムで使用する変数定義
//==============================================================================

uint8_t iLoop1s = 0;							// 定期的に処理をおこなうためのカウンタ
volatile bool bInterval = false;	// タイマ割り込みをメインループで処理するためのフラグ
																	// 割り込み処理に関わる変数は固定（volatile）

int	axisX_g, axisY_g, axisZ_g;		// 加速度センサの値
int cnt=0;												// デバッグ用


//==============================================================================
// プログラム
//==============================================================================

//------------------------------------------------------------------------------
// setup / 初期化
//------------------------------------------------------------------------------

//---------------------------------------------------------------------
// IOピンの入出力設定
//---------------------------------------------------------------------
void setupPort()
{
  pinMode(BLE_RESET_PIN, OUTPUT);			// BLEリセット端子
  digitalWrite(BLE_RESET_PIN, LOW);

  pinMode(BLE_WAKEUP_PIN, OUTPUT); 		// BLEウェイクアップ端子
  digitalWrite(BLE_WAKEUP_PIN, HIGH);

  pinMode(LED_PIN, OUTPUT); 					// LED端子（おまけ）
  digitalWrite(LED_PIN, LOW);					// ビーコン発信が分かるように。

}


//---------------------------------------------------------------------
// 割り込み処理の初期設定
//---------------------------------------------------------------------

//-----------------------------------------------
// タイマ割込み
// ・割込み間隔：125ms
// ・割込みはオーバーフロー
// メインループのタイマー割り込み設定
//-----------------------------------------------
void setupTC2Int()
{
  MsTimer2::set(LOOP_INTERVAL, intTimer2);
}


//---------------------------------------------------------------------
// 各デバイスの初期設定
//---------------------------------------------------------------------

//-----------------------------------------------
// センサの初期化
//-----------------------------------------------
void setupSensor()
{
  accel.begin(LIS2DH_ADDRESS);
  accel.setClick(0, 0);                     // Disable Interrupt
  accel.setRange(LIS3DH_RANGE_2_G);         // Full scale +/- 2G
  accel.setDataRate(LIS3DH_DATARATE_1_HZ);  // Data rate = 1Hz
}


//-----------------------------------------------
// BLEの初期化
//-----------------------------------------------
void setupBLE()
{

  // 内部状態ハンドラの登録（ 発信時LEDを点灯するためのおまけ ）
  ble112.onBusy = onBusy;
  ble112.onIdle = onIdle;

  // BGLib イベントハンドラの登録
  ble112.ble_rsp_system_get_bt_address = my_rsp_system_get_bt_address;

	// BLE用のUART設定
  Serialble.begin(9600);

	// アドバタイズのパラメータ設定
  // interval_min : 250ms( = 400 x 0.625ms )
  // interval_max : 500ms( = 800 x 0.625ms )
	// <interval_min> <interval_max> <channel_map>
  ble112.ble_cmd_le_gap_set_adv_parameters(400, 800, 7); 
  while (ble112.checkActivity(1000)); 	// 受信チェック

}


//------------------------------------------------------------------------------
// その他関数
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// 整数型データを送信文字列に変換
// param     : uint8_t *upperVal	upper byte 送信データ上位
//           : uint8_t *lowerVal	lower byte 送信データ下位
//           : uint16_t sourceVal 2bytes　送信したいデータ
//------------------------------------------------------------------------------
void convUint8fromInt16( uint8_t *upperVal, uint8_t *lowerVal, uint16_t sourceVal ){

  *upperVal = (sourceVal >> 8) & 0x0FF;
  *lowerVal = sourceVal & 0x0FF;

}


//------------------------------------------------------------------------------
// 受信文字列を整数型データに変換
// param     : uint8_t upperVal	upper byte 受信データ上位
//           : uint8_t lowerVal	lower byte 受信データ下位
// return    : int message value 4bytes 受信したデータ
//------------------------------------------------------------------------------
int convInt16fromInt8( uint8_t upperVal, uint8_t lowerVal ){

  return (upperVal << 8) |(lowerVal & 0x0FF);

}


//------------------------------------------------------------------------------
// アドバタイズ（ビーコン発信）する本体部分
//------------------------------------------------------------------------------
void StartAdvData()
{

  //---- 送信文字列の準備 ----
	// アドバタイジングデータは最大25バイト
  uint8_t adv_data[] = {

    // AD Structure 1: Flag
    (2),  	//0: field length
    BGLIB_GAP_AD_TYPE_FLAGS,  //1: field type (0x01)
    (6),  	//2: data

    // AD Structure 2: Complete local name
    (7),  	//3: field length
    BGLIB_GAP_AD_TYPE_LOCALNAME_COMPLETE,  //4: field type (0x09)
    ('-'),  //5: local name (MSB)
    ('-'),  //6:
    ('-'),  //7:
    ('-'),  //8:
    ('-'),  //9:
    ('-'),  //10: local name (LSB)

    // AD Structure 3: Manufacture specific
    (13),   //11: field lengt　h
    (0xff), //12-1: field type (0xff=Manufacture Specific)
    (0xff), //13-2: Company ID
    (0xff), //14-3:   (Be used in the internal and interoperability tests.)
    ('3'),  //15-4: cnt upper
    ('4'),  //16-5: cnt lower
    ('5'),  //17-6: axisX_g
    ('6'),  //18-7:
    ('7'),  //19-8: axisY_g
    ('8'),  //20-9:
    ('9'),	//21-10: axisZ_g
    ('A'), 	//22-11:
    ('-'),  //23-12:
    ('-')   //24-13:
  };

	//---- デバイス名を送信文字列に入れる ----
  for (int i = 0; i < 6; i++)
  {
		adv_data[5+i] = strDeviceName[i];
	}

	//----- 送信データを送信文字列に入れる ----
	convUint8fromInt16( &adv_data[15], &adv_data[16], cnt );
	convUint8fromInt16( &adv_data[17], &adv_data[18], axisX_g );
	convUint8fromInt16( &adv_data[19], &adv_data[20], axisY_g );
	convUint8fromInt16( &adv_data[21], &adv_data[22], axisZ_g );

	#ifdef DEBUG														// センサの送信データ
	  Serial.println(">>>> Tx data");
	  for(int i = 0; i < 24; i++){
	    Serial.print(adv_data[i],HEX);
	  }
	  Serial.println("");
	#endif

  //---- アドバタイズデータを登録 ---
	// 送信文字列長は25に固定
  ble112.ble_cmd_le_gap_set_adv_data(SCAN_RSP_ADVERTISING_PACKETS, 25, adv_data);
  while (ble112.checkActivity(1000));  		// 受信チェック

  //---- アドバタイザを送信 ---- 
  ble112.ble_cmd_le_gap_start_advertising(0, LE_GAP_USER_DATA, LE_GAP_SCANNABLE_NON_CONNECTABLE);
  while (ble112.checkActivity(1000));  		// 受信チェック
}

//------------------------------------------------------------------------------
// 割り込み処理
//------------------------------------------------------------------------------

//----------------------------------------------
// タイマー割り込み関数
// Timer2 INT
//----------------------------------------------
void intTimer2()
{
  bInterval = true;												// ループで処理するためにフラグで伝える
}



//------------------------------------------------------------------------------
// setup / 初期化
//------------------------------------------------------------------------------
void setup()
{

  Serial.begin(115200);										// プログラム書き込み・デバッグ用通信
  delay(10);															// 少し待つと安定する（MCUによってはこれが必要）
  Wire.begin();														// I2C 100KHz（センサ用）

	//---- ポート初期化 ----
  setupPort();							
  delay(10);

	//---- タイマ割込み初期化 ----
  noInterrupts();													// 設定中は割込み禁止
  setupTC2Int();
  interrupts();

	//---- センサ初期化 ----
	setupSensor();

	//---- BLE初期化 ----
  setupBLE();

  ble112.ble_cmd_system_get_bt_address();	// アドレスの取得
  while (ble112.checkActivity(1000));			// 実際の取得はコールバックで
  delay(1000);

  MsTimer2::start();											// タイマ割込み開始

}


//------------------------------------------------------------------------------
// loop / メインループ
//------------------------------------------------------------------------------
void loop()
{

  if (bInterval == true)
  {														// タイマー割り込みがあった
    bInterval = false;
	
	  iLoop1s += 1;							// アドバタイザの発信間隔を制御
	  if (iLoop1s >= 16)
	  { 												// 125ms x 8 x 2 = 2s
	    iLoop1s = 0;

				++cnt;								// ループ回数

				//---- センサ値の読み取り ----
		    accel.read();																

				//---- センサ値を整数化 ----
		    axisX_g = (uint16_t)( accel.x_g * 100 );
		    axisY_g = (uint16_t)( accel.y_g * 100 );
		    axisZ_g = (uint16_t)( accel.z_g * 100 );


				#ifdef DEBUG
			    Serial.println("");
			    Serial.println("");
			    Serial.print(String(cnt));
			    Serial.print(", ");
			    Serial.print(String(axisX_g));
			    Serial.print(", ");
			    Serial.print(String(axisY_g));
			    Serial.print(", ");
			    Serial.print(String(axisZ_g));
			    Serial.println("");
				#endif

				//---- アドバタイザ送信 ----
				StartAdvData();

	  }
  }
}


//------------------------------------------------------------------------------
// BLEのコールバック関数
// イベント発生時にライブラリから呼ばれる関数
// https://static4.arrow.com/-/media/arrow/files/pdf/s/silicon-experts_api-ref-guide_blue-gecko-article.pdf
//------------------------------------------------------------------------------

//----------------------------------------------
// モジュールがコマンドの送信を開始する
//----------------------------------------------
void onBusy()
{
  digitalWrite( LED_PIN, HIGH );				  // 動作確認用LEDの点灯（おまけ）
}

//----------------------------------------------
// モジュールが完全な応答または「system_boot」イベントを受信したとき
//----------------------------------------------
void onIdle()
{
  digitalWrite( LED_PIN, LOW );				 	 // 動作確認用LEDの消灯（おまけ）
}

//----------------------------------------------
// アドレス取得時にデバッグ出力（おまけ）
//----------------------------------------------
void my_rsp_system_get_bt_address(const struct ble_msg_system_get_bt_address_rsp_t *msg)
{

	#ifdef DEBUG
	  Serial.print("###\tsystem_get_bt_address: { ");
	  Serial.print("address: ");
	  for (int i = 0; i < 6; i++)
	  {
	    Serial.print(msg->address.addr[i], HEX);
	  }
	  Serial.println(" }");
	#endif

}


//==============================================================================
// End of code
